/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.fpmislata.persistencia.hibernate.util;

import java.io.Serializable;
import java.sql.Connection;
import java.util.Map;
import java.util.Set;
import javax.naming.NamingException;
import javax.naming.Reference;
import org.hibernate.*;
import org.hibernate.engine.spi.FilterDefinition;
import org.hibernate.metadata.ClassMetadata;
import org.hibernate.metadata.CollectionMetadata;
import org.hibernate.stat.Statistics;

class SessionFactoryImplThreadLocal implements SessionFactory {

    private HibernateUtilInternalState state;
    
    /**
     * Nos retorna la sesión que está asociada al Thread
     * @return Session asociada al Thread
     * @throws HibernateException 
     */
    @Override
    public Session getCurrentSession() throws HibernateException {
        return state.threadLocalSession.get();
    }    
    
    public SessionFactoryImplThreadLocal(HibernateUtilInternalState state) {
        this.state=state;
    }

    @Override
    public SessionFactoryOptions getSessionFactoryOptions() {
        return state.sessionFactory.getSessionFactoryOptions();
    }

    @Override
    public SessionBuilder withOptions() {
        return state.sessionFactory.withOptions();
    }

    @Override
    public Session openSession() throws HibernateException {
        return state.sessionFactory.openSession();
    }

    @Override
    public StatelessSessionBuilder withStatelessOptions() {
        return state.sessionFactory.withStatelessOptions();
    }

    @Override
    public StatelessSession openStatelessSession() {
        return state.sessionFactory.openStatelessSession();
    }

    @Override
    public StatelessSession openStatelessSession(Connection cnctn) {
        return state.sessionFactory.openStatelessSession(cnctn);
    }

    @Override
    public ClassMetadata getClassMetadata(Class type) {
        return state.sessionFactory.getClassMetadata(type);
    }

    @Override
    public ClassMetadata getClassMetadata(String string) {
        return state.sessionFactory.getClassMetadata(string);
    }

    @Override
    public CollectionMetadata getCollectionMetadata(String string) {
        return state.sessionFactory.getCollectionMetadata(string);
    }

    @Override
    public Map<String, ClassMetadata> getAllClassMetadata() {
        return state.sessionFactory.getAllClassMetadata();
    }

    @Override
    public Map getAllCollectionMetadata() {
        return state.sessionFactory.getAllCollectionMetadata();
    }

    @Override
    public Statistics getStatistics() {
        return state.sessionFactory.getStatistics();
    }

    @Override
    public void close() throws HibernateException {
        state.sessionFactory.close();
    }

    @Override
    public boolean isClosed() {
        return state.sessionFactory.isClosed();
    }

    @Override
    public Cache getCache() {
        return state.sessionFactory.getCache();
    }

    @Override
    public void evict(Class type) throws HibernateException {
        state.sessionFactory.evict(type);
    }

    @Override
    public void evict(Class type, Serializable srlzbl) throws HibernateException {
        state.sessionFactory.evict(type,srlzbl);
    }

    @Override
    public void evictEntity(String string) throws HibernateException {
        state.sessionFactory.evictEntity(string);
    }

    @Override
    public void evictEntity(String string, Serializable srlzbl) throws HibernateException {
        state.sessionFactory.evictEntity(string,srlzbl);
    }

    @Override
    public void evictCollection(String string) throws HibernateException {
        state.sessionFactory.evictCollection(string);
    }

    @Override
    public void evictCollection(String string, Serializable srlzbl) throws HibernateException {
        state.sessionFactory.evictCollection(string,srlzbl);
    }

    @Override
    public void evictQueries(String string) throws HibernateException {
        state.sessionFactory.evictQueries(string);
    }

    @Override
    public void evictQueries() throws HibernateException {
        state.sessionFactory.evictQueries();
    }

    @Override
    public Set getDefinedFilterNames() {
        return state.sessionFactory.getDefinedFilterNames();
    }

    @Override
    public FilterDefinition getFilterDefinition(String string) throws HibernateException {
        return state.sessionFactory.getFilterDefinition(string);
    }

    @Override
    public boolean containsFetchProfileDefinition(String string) {
        return state.sessionFactory.containsFetchProfileDefinition(string);
    }

    @Override
    public TypeHelper getTypeHelper() {
        return state.sessionFactory.getTypeHelper();
    }

    @Override
    public Reference getReference() throws NamingException {
        return state.sessionFactory.getReference();
    }
    
}
